#!/usr/bin/perl -w

use POSIX;
use strict;

my $spr2pc = `which spr2pc` || die "can't find spr2pc";
chomp($spr2pc);
my $filename = $ARGV[0] || '/home/fft/cd/us/open/opntex.bin';
my $output = $ARGV[1] || 'opntex{}.png';
my @extra_options = @ARGV[1 .. $#ARGV];
my $size = (-s $filename);

open(DATA, "<$filename") || die "can't open $filename";
my @offs = (0,34816,69632,137216,172032,206848,241664,288768,335872,382976,430080,477184,524288,571392,618496,665600,712704,759808,806912,854016,901120,948224);

my $count = 0;
my $offset = 0;
while ($offset < $size) {
  #
  # Layout for opntex.bin
  #
  # Type Count Description Modifies
  # 1. u32 1 header size	#2
  # 2. u16 V header bytes	#3, #5
  # 3. u16 V palette bytes
  # 4. u32 1 image size		#6
  # 5. u16 V footer bytes
  # 6. u 8 V image bytes
  # 7. Padding to 2,048 byte block size.

  sysseek(DATA, $offset, SEEK_SET);

  my $header_byte_tmp;
  sysread(DATA, $header_byte_tmp, 4);	# Always 16 in opntex.bin
  my $header_byte_size = unpack("L", $header_byte_tmp);
  printf("%02d:%06d:hz: hbs[%10lu]\n", $count, $offset, $header_byte_size);

  # All sizes relative to start of header itself.
  $offset += 4;

  # -----

  my $header;
  sysread(DATA, $header, $header_byte_size);
  my @header = unpack("LS*", $header);
  printf("%02d:%06d:hb: " . join(' ', ('%5u') x (scalar @header)) . "\n", $count, $offset, @header);

  my $footer_byte_size   = $header[0];
  my $footer_byte_offset = $header[1];
  my $zero_h1            = $header[2];
  my $zero_h2            = $header[3];
  my $foureightyish      = $header[4];
  my $palette_colors     = $header[5];
  my $palette_alternates = $header[6];

  # This is always 480-483 for opntex.bin. No idea what it does. Screen mode?
  die "fourtyeightish: $foureightyish" if (($foureightyish & ~3) != 480);
  die "failed zero assert: 1=$zero_h1 2=$zero_h2" if ($zero_h1 || $zero_h2);
  # -----

  # Now what is this +4 for? The footer byte size?
  # But that screws up other offset values.
  sysseek(DATA, $offset + $footer_byte_offset + 4, SEEK_SET);

  my $image_byte_tmp;
  sysread(DATA, $image_byte_tmp, 4);
  my $image_byte_size = unpack('L', $image_byte_tmp);

  my $footer;
  sysread(DATA, $footer, $footer_byte_size);
  my @footer = unpack("S*C", $footer);
  printf("%02d:%06d:fb: ibs[%10lu]" . join(' ', ('%5u') x (scalar @footer)) . "\n", $count, $offset, $image_byte_size, @footer);

  my $unknown_value     = $footer[0];	# 256, 320, 512, 768, 832, 896
  my $zero_f1           = $footer[1];	# Usually 0. 256="Slow partial display"?
  my $image_byte_width  = $footer[2];
  my $image_byte_height = $footer[3];
  my $maybe_flags       = $footer[4];	# 0, 128, 255 -- flags?

#  die "failed zero assert: 1=$zero_f1" if ($zero_f1);

  # -----

  # The +6 and ++ are too magical for my liking.
  my $pictoff = $offset + $footer_byte_offset + $footer_byte_size + 6;
  $pictoff++ if ($footer_byte_size == 9);

  my $palette_offset = $offset + $header_byte_size;
  my $splitvalue = $image_byte_width * 2;
  my $palbytes = $palette_colors * 2;
  my $palsize = $palette_colors == 256 ? 0 : 16;
  my $pictsize = $image_byte_size;
#  $pictsize += 65536 if ($pictsize < 32768);

  # Next image is: Header + Palette + Image + Footer
  $offset += 20 + $palbytes + $pictsize + 12;
  # Then block-align it for the CD-ROM (2048 byte sectors);
  $offset = ($offset + 2047) & ~2047;
  if ($count < $#offs && $offset != $offs[$count + 1]) {
    print "$offset (" . $offs[$count + 1] . ")\n";
    exit;
  }

  for (my $use_palette = 0; $use_palette < $palette_alternates; $use_palette++) {
    my $out;
    ($out = $output) =~ s/{}/sprintf("%02d-%02d", $count, $use_palette)/e;
    my $exec = "$spr2pc " .
	"-z 1:1:1 " .
	"-l $splitvalue " .
	"-a $palette_offset " .
	"-j $pictoff:$pictsize " .
	"-r 0:$palbytes:$palsize " .
	"-o $out " .
	"$filename " .
	"@extra_options";
    system($exec);

    $palette_offset += $palbytes;
  }
  $count++;
}
