#!/usr/bin/perl -w

use POSIX;
use strict;

my $spr2pc = `which spr2pc` || die "can't find spr2pc";
chomp($spr2pc);
my $filename = shift || '/home/fft/cd/us/world/wldpic.bin';
my $output = shift || 'wldpic{}.png';
my $offset = shift || 0;
my $size = (-s $filename);

open(DATA, "<$filename") || die "can't open $filename";

my $count = 0;
while ($offset < $size) {
  print '[', (sprintf '%10d', $offset);
  sysseek(DATA, $offset, SEEK_SET);

  my $header_bytes;
  sysread(DATA, $header_bytes, 4);
  $header_bytes = unpack('L', $header_bytes);

  if ($header_bytes <= 0 || $header_bytes > 128) {
    print ": $header_bytes\n";
    sysread(DATA, $count, 1024);
    print join(' ', unpack('C*', $count)), "\n";
    exit;
  }
  $offset += 4;

  my $header;
  sysread(DATA, $header, $header_bytes);

  my @header = unpack('L2S' . $header_bytes, $header);
  my $footer_bytes  = $header[0];
  my $footer_offset = $header[1];
  my $unknown_0 = $header[2];	# 0
  my $unknown_1 = $header[3];	# 480-483
  my $pal_bytes = $header[4];	# size of each palette
  my $pal_number = $header[5];	# number of palettes

  $footer_bytes = 8 if ($footer_bytes > 128);
  my $pal_offset = $offset + $header_bytes;

  sysseek(DATA, $offset + $footer_offset + 4, SEEK_SET);

  my $data_bytes;
  sysread(DATA, $data_bytes, 4);
  $data_bytes = unpack('L', $data_bytes);

  my $footer;
  sysread(DATA, $footer, $footer_bytes);
  my @footer = unpack($footer_bytes % 2 == 1 ? 'S*C' : 'S*', $footer);

  my $img_horiz = $footer[2];
  my $img_vert = $footer[3];
  my $img_bytes = $data_bytes - $footer_bytes - 4;
  my $img_offset = $offset + $footer_offset + $footer_bytes + 8;

  # This is rounded to multiples of 2,048.  It's the block size of
  # a CD-ROM if you're wondering.
  my $this_bytes = 4 + $footer_offset + $data_bytes;
  $this_bytes = (($this_bytes + 2047) & 0xffff800) - 4;

  print ": $header_bytes|@header|$img_bytes|@footer|$this_bytes]\n";
  $offset += $this_bytes;

  my $out;
  ($out = $output) =~ s/{}/sprintf '%02d', $count++/e;

  my $img_width = $img_horiz * 2;
  my $pal_size = $pal_bytes == 256 ? 0 : $pal_bytes;
  $pal_bytes *= 2;

  my $exec = join(' ',
		$spr2pc,
		"-l $img_width",
		"-a $pal_offset",
		"-j $img_offset:$img_bytes",
		$filename,
		"-o $out",
		"-r 0:$pal_bytes:$pal_size",
		@ARGV,
		'> /dev/null'
		);
#  print $exec, "\n";
  system($exec);
}
