void error(const char *msg);

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <fcntl.h>
#include "mmaplib.h"

/*
 * Pre-process wldtex.tm2 to fix it.
 */

struct code_list {
  int match_len;
  int skip_len;
  const unsigned char bstr[8];
};
/*
  { 5, 12, {   2, 0,  0, 0,  64	} },
  { 5, 12, {   2, 0,  0, 0, 220	} },
  { 5, 12, {   2, 0,  0, 0,  14	} },
  { 4, 12, {   3, 0,  0, 0	} },
  { 2,  8, { 128, 3		} },
  { 2,  8, {   0, 3		} },
*/
struct {
  int width;
  int height;
  const char *name;
  struct code_list codes[32];
} files[] = {
  { 240, 240, "/tmp/map-00.wldtexdata", {
    /* match_len  skip_len  { bytes } */
    { 5, 12, { 2, 0, 0, 0, 64	} },
    { 4, 12, { 3, 0, 0, 0	} },
    { 2,  8, { 0, 3		} },
  } },

  { 256, 240, "/tmp/map-01.wldtexdata", {
    { 5, 12, { 2, 0, 0, 0, 220	} },
    { 4, 12, { 3, 0, 0, 0	} },
    { 2,  8, { 128, 3		} },
  } },

  { 240, 128, "/tmp/map-02.wldtexdata", {
    { 5, 12, { 2, 0, 0, 0, 14	} },
    { 4, 12, { 3, 0, 0, 0	} },
    { 2,  8, { 0, 1		} },
  } },

  { 256, 130, "/tmp/map-03.wldtexdata", {
    { 5, 12, { 2, 0, 0, 0, 32	} },
    { 4, 12, { 3, 0, 0, 0	} },
    { 2,  8, { 0, 1		} },
  } },

  { 128, 256, "/tmp/map-04.wldtexdata", {
    { 5, 12, {   2, 0, 0, 0, 224	} },	/* v/ */
    { 5, 12, {   3, 0, 0, 0, 252	} },	/* v/ */
    { 5, 12, {   3, 0, 0, 0, 250	} },	/* v/ */
    { 5, 12, {   3, 0, 0, 0, 244	} },	/* v/ */
    { 5, 12, {   3, 0, 0, 0, 242	} },	/* v/ */
    { 5, 12, {   3, 0, 0, 0, 238	} },	/* v/ */
    { 5, 12, {   3, 0, 0, 0, 236	} },	/* v/ */
    { 5, 12, {   3, 0, 0, 0, 230	} },	/* v/ */
    { 5, 12, {   3, 0, 0, 0, 228	} },	/* v/ */
    { 5, 12, {   3, 0, 0, 0, 222	} },	/* v/ */
    { 5, 12, {   3, 0, 0, 0, 216	} },	/* v/ */
    { 5, 12, {   3, 0, 0, 0, 214	} },	/* v/ */
    { 5, 12, {   3, 0, 0, 0, 208	} },	/* v/ */
    { 5, 12, {   3, 0, 0, 0, 202	} },	/* v/ */
    { 5, 12, {   3, 0, 0, 0, 200	} },	/* v/ */
    { 5, 12, {   3, 0, 0, 0, 194	} },	/* v/ */
    { 2,  8, { 192, 1			} },	/* v/ */
  } },

  /* Data below this point in the file is not a map. */
};

const char *default_wldtex = "/home/fft/cd/us/world/wldtex.tm2";

int main(int argc, char *argv[])
{
  void *original;
  const char *filename;
  int outfd, img;
  size_t user_offset;

  filename = argc < 2 ? default_wldtex : argv[1];
  user_offset = argc < 3 ? 0 : atol(argv[2]);
  if (!(original = mapopen(filename, O_RDONLY)))
    error("mapopen read");

  for (img = 0; img < sizeof(files) / sizeof(*files); img++) {
    void *fiximg;
    unsigned char *fixdata, *origdata;
    int	sbyte, dbyte,
	headerbytes = 20,
	palettebytes = 512,
	databytes = files[img].width * files[img].height,
	totalbytes = headerbytes + palettebytes + databytes;

    if ((outfd = open(files[img].name, O_RDWR | O_CREAT | O_TRUNC, 0777)) < 0)
      error("open");
    if (ftruncate(outfd, totalbytes) < 0) /* tmp fudge factor */
      error("ftruncate");
    close(outfd);
    if (!(fiximg = mapopen(files[img].name, O_RDWR)))
      error("mapopen write");

    printf("%d\n", files[img].width);
    fprintf(stderr, "%s: %3d/%3d: %u\n", files[img].name, files[img].width, files[img].height, user_offset);

    memcpy(mapdata(fiximg), mapdata(original) + user_offset, headerbytes + palettebytes);
    fixdata = mapdata(fiximg) + headerbytes + palettebytes;
    origdata = mapdata(original) + headerbytes + palettebytes + user_offset;

    sbyte = dbyte = 0;
    while (dbyte < databytes && user_offset < maplength(original)) {
      int mt, found;
      struct code_list *codes = files[img].codes;

      found = 0;
      for (mt = 0; codes[mt].match_len; mt++)
        if (memcmp(origdata + sbyte, codes[mt].bstr, codes[mt].match_len) == 0) {
          sbyte += codes[mt].skip_len;
          found = 1;
          break;
        }

      if (!found)
        fixdata[dbyte++] = origdata[sbyte++];
    }
    user_offset += totalbytes;
    user_offset = (user_offset + 2047) & ~2047;
  }
  mapclose(original);
  return 0;
}
